@@ -83,51 +83,8 @@
 #'     nanonext website: \url{https://shikokuchuo.net/nanonext/} \cr
 #'     nanonext on CRAN: \url{https://cran.r-project.org/package=nanonext}
 #'
-#'     NNG website: \url{https://nng.nanomsg.org/}
-#'
-#' @section Licence notices:
-#'
-#'     Copyright (C) 2022 Hibiki AI Limited <\email{info@@hibiki-ai.com}>
-#'
-#'     This file is a part of nanonext.
-#'
-#'     nanonext is free software: you can redistribute it and/or modify it under
-#'     the terms of the GNU General Public License as published by the Free
-#'     Software Foundation, either version 3 of the License, or (at your option)
-#'     any later version.
-#'
-#'     nanonext is distributed in the hope that it will be useful, but WITHOUT
-#'     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-#'     FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-#'     more details.
-#'
-#'     You should have received a copy of the GNU General Public License along
-#'     with nanonext. If not, see <https://www.gnu.org/licenses/>.
-#'
-#'     nanonext links to the NNG library with the following licence:
-#'
-#'     The MIT License
-#'
-#'     Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
-#'     Copyright 2018 Capitar IT Group BV <info@capitar.com>
-#'
-#'     Permission is hereby granted, free of charge, to any person obtaining a copy
-#'     of this software and associated documentation files (the "Software"),
-#'     to deal in the Software without restriction, including without limitation
-#'     the rights to use, copy, modify, merge, publish, distribute, sublicense,
-#'     and/or sell copies of the Software, and to permit persons to whom
-#'     the Software is furnished to do so, subject to the following conditions:
-#'
-#'     The above copyright notice and this permission notice shall be included
-#'     in all copies or substantial portions of the Software.
-#'
-#'     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-#'     IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-#'     FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-#'     THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-#'     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-#'     FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
-#'     IN THE SOFTWARE.
+#'     NNG website: \url{https://nng.nanomsg.org/} \cr
+#'     Mbed TLS website: \url{https://www.trustedfirmware.org/projects/mbed-tls/}
 #'
 #' @encoding UTF-8
 #' @author Charlie Gao \email{charlie.gao@@shikokuchuo.net}
@@ -143,7 +100,7 @@
 NULL
 
 .onUnload <- function(libpath) {
-  if (!is.null(warn <- getOption("nanonext.original.warn"))) {
+  if (length(warn <- getOption("nanonext.original.warn"))) {
     options(warn = warn)
     options(nanonext.original.warn = NULL)
   }
